package java.lang;

import java.util.Objects;
import java.util.function.Supplier;
import java.util.ResourceBundle;
import java.security.AccessControlContext;
import sun.nio.ch.Interruptible;
import java.lang.reflect.Executable;
import java.lang.annotation.Annotation;
import sun.reflect.annotation.AnnotationType;
import sun.reflect.ConstantPool;
import sun.misc.JavaLangAccess;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.FileDescriptor;
import sun.misc.Version;
import sun.misc.VM;
import java.io.UnsupportedEncodingException;
import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import sun.reflect.CallerSensitive;
import sun.reflect.Reflection;
import java.util.Map;
import java.util.PropertyPermission;
import java.security.AccessController;
import sun.security.util.SecurityConstants;
import java.security.PrivilegedAction;
import java.io.IOException;
import java.nio.channels.spi.SelectorProvider;
import java.nio.channels.Channel;
import sun.misc.SharedSecrets;
import java.util.Properties;
import java.io.Console;
import java.io.PrintStream;
import java.io.InputStream;

public final class System
{
    public static final InputStream in;
    public static final PrintStream out;
    public static final PrintStream err;
    private static volatile SecurityManager security;
    private static volatile Console cons;
    private static Properties props;
    private static String lineSeparator;
    
    private static native void registerNatives();
    
    private System() {
    }
    
    public static void setIn(InputStream in0) {
        checkIO();
        setIn0(in0);
    }
    
    public static void setOut(PrintStream out0) {
        checkIO();
        setOut0(out0);
    }
    
    public static void setErr(PrintStream err0) {
        checkIO();
        setErr0(err0);
    }
    
    public static Console console() {
        if (System.cons == null) {
            synchronized (System.class) {
                System.cons = SharedSecrets.getJavaIOAccess().console();
            }
        }
        return System.cons;
    }
    
    public static Channel inheritedChannel() throws IOException {
        return SelectorProvider.provider().inheritedChannel();
    }
    
    private static void checkIO() {
        SecurityManager securityManager = getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new RuntimePermission("setIO"));
        }
    }
    
    private static native void setIn0(InputStream p0);
    
    private static native void setOut0(PrintStream p0);
    
    private static native void setErr0(PrintStream p0);
    
    public static void setSecurityManager(SecurityManager securityManager0) {
        try {
            securityManager0.checkPackageAccess("java.lang");
        }
        catch (Exception ex) {}
        setSecurityManager0(securityManager0);
    }
    
    private static synchronized void setSecurityManager0(final SecurityManager security) {
        SecurityManager securityManager = getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new RuntimePermission("setSecurityManager"));
        }
        if (security != null && security.getClass().getClassLoader() != null) {
            AccessController.doPrivileged(new PrivilegedAction<Object>() {
                @Override
                public Object run() {
                    security.getClass().getProtectionDomain().implies(SecurityConstants.ALL_PERMISSION);
                    return null;
                }
            });
        }
        System.security = security;
    }
    
    public static SecurityManager getSecurityManager() {
        return System.security;
    }
    
    public static native long currentTimeMillis();
    
    public static native long nanoTime();
    
    public static native void arraycopy(Object p0, int p1, Object p2, int p3, int p4);
    
    public static native int identityHashCode(Object p0);
    
    private static native Properties initProperties(Properties p0);
    
    public static Properties getProperties() {
        SecurityManager securityManager = getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPropertiesAccess();
        }
        return System.props;
    }
    
    public static String lineSeparator() {
        return System.lineSeparator;
    }
    
    public static void setProperties(Properties props) {
        SecurityManager securityManager = getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPropertiesAccess();
        }
        if (props == null) {
            props = new Properties();
            initProperties(props);
        }
        System.props = props;
    }
    
    public static String getProperty(String key) {
        checkKey(key);
        SecurityManager securityManager = getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPropertyAccess(key);
        }
        return System.props.getProperty(key);
    }
    
    public static String getProperty(String key, String defaultValue) {
        checkKey(key);
        SecurityManager securityManager = getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPropertyAccess(key);
        }
        return System.props.getProperty(key, defaultValue);
    }
    
    public static String setProperty(String s, String value) {
        checkKey(s);
        SecurityManager securityManager = getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new PropertyPermission(s, "write"));
        }
        return (String)System.props.setProperty(s, value);
    }
    
    public static String clearProperty(String s) {
        checkKey(s);
        SecurityManager securityManager = getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new PropertyPermission(s, "write"));
        }
        return (String)System.props.remove(s);
    }
    
    private static void checkKey(String s) {
        if (s == null) {
            throw new NullPointerException("key can't be null");
        }
        if (s.equals("")) {
            throw new IllegalArgumentException("key can't be empty");
        }
        return;
    }
    
    public static String getenv(String s) {
        SecurityManager securityManager = getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new RuntimePermission("getenv." + s));
        }
        return ProcessEnvironment.getenv(s);
    }
    
    public static Map<String, String> getenv() {
        SecurityManager securityManager = getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new RuntimePermission("getenv.*"));
        }
        return ProcessEnvironment.getenv();
    }
    
    public static void exit(int n) {
        Runtime.getRuntime().exit(n);
    }
    
    public static void gc() {
        Runtime.getRuntime().gc();
    }
    
    public static void runFinalization() {
        Runtime.getRuntime().runFinalization();
    }
    
    @Deprecated
    public static void runFinalizersOnExit(boolean b) {
        Runtime.runFinalizersOnExit(b);
    }
    
    @CallerSensitive
    public static void load(String s) {
        Runtime.getRuntime().load0(Reflection.getCallerClass(), s);
    }
    
    @CallerSensitive
    public static void loadLibrary(String s) {
        Runtime.getRuntime().loadLibrary0(Reflection.getCallerClass(), s);
    }
    
    public static native String mapLibraryName(String p0);
    
    private static PrintStream newPrintStream(FileOutputStream fileOutputStream, String encoding) {
        if (encoding != null) {
            try {
                return new PrintStream(new BufferedOutputStream(fileOutputStream, 128), true, encoding);
            }
            catch (UnsupportedEncodingException ex) {}
        }
        return new PrintStream(new BufferedOutputStream(fileOutputStream, 128), true);
    }
    
    private static void initializeSystemClass() {
        initProperties(System.props = new Properties());
        VM.saveAndRemoveProperties(System.props);
        System.lineSeparator = System.props.getProperty("line.separator");
        Version.init();
        FileInputStream in = new FileInputStream(FileDescriptor.in);
        FileOutputStream fileOutputStream = new FileOutputStream(FileDescriptor.out);
        FileOutputStream fileOutputStream2 = new FileOutputStream(FileDescriptor.err);
        setIn0(new BufferedInputStream(in));
        setOut0(newPrintStream(fileOutputStream, System.props.getProperty("sun.stdout.encoding")));
        setErr0(newPrintStream(fileOutputStream2, System.props.getProperty("sun.stderr.encoding")));
        loadLibrary("zip");
        Terminator.setup();
        VM.initializeOSEnvironment();
        Thread currentThread = Thread.currentThread();
        currentThread.getThreadGroup().add(currentThread);
        setJavaLangAccess();
        VM.booted();
    }
    
    private static void setJavaLangAccess() {
        SharedSecrets.setJavaLangAccess(new JavaLangAccess() {
            public ConstantPool getConstantPool(Class<?> clazz) {
                return clazz.getConstantPool();
            }
            
            public boolean casAnnotationType(Class<?> clazz, AnnotationType annotationType, AnnotationType annotationType2) {
                return clazz.casAnnotationType(annotationType, annotationType2);
            }
            
            public AnnotationType getAnnotationType(Class<?> clazz) {
                return clazz.getAnnotationType();
            }
            
            public Map<Class<? extends Annotation>, Annotation> getDeclaredAnnotationMap(Class<?> clazz) {
                return clazz.getDeclaredAnnotationMap();
            }
            
            public byte[] getRawClassAnnotations(Class<?> clazz) {
                return clazz.getRawAnnotations();
            }
            
            public byte[] getRawClassTypeAnnotations(Class<?> clazz) {
                return clazz.getRawTypeAnnotations();
            }
            
            public byte[] getRawExecutableTypeAnnotations(Executable executable) {
                return Class.getExecutableTypeAnnotationBytes(executable);
            }
            
            public <E extends Enum<E>> E[] getEnumConstantsShared(Class<E> clazz) {
                return (E[])clazz.getEnumConstantsShared();
            }
            
            public void blockedOn(Thread thread, Interruptible interruptible) {
                thread.blockedOn(interruptible);
            }
            
            public void registerShutdownHook(int n, boolean b, Runnable runnable) {
                Shutdown.add(n, b, runnable);
            }
            
            public int getStackTraceDepth(Throwable t) {
                return t.getStackTraceDepth();
            }
            
            public StackTraceElement getStackTraceElement(Throwable t, int n) {
                return t.getStackTraceElement(n);
            }
            
            public String newStringUnsafe(char[] array) {
                return new String(array, true);
            }
            
            public Thread newThreadWithAcc(Runnable runnable, AccessControlContext accessControlContext) {
                return new Thread(runnable, accessControlContext);
            }
            
            public void invokeFinalize(Object o) throws Throwable {
                o.finalize();
            }
        });
    }
    
    static {
        registerNatives();
        in = null;
        out = null;
        err = null;
        System.security = null;
        System.cons = null;
    }
    
    public interface Logger
    {
        public abstract String getName();
        
        public abstract boolean isLoggable(System.Logger.Level p0);
        
        default public void log(System.Logger.Level level, String msg) {
            this.log(level, (ResourceBundle)null, msg, (Object[])null);
        }
        
        default public void log(System.Logger.Level level, Supplier<String> msgSupplier) {
            Objects.requireNonNull(msgSupplier);
            if (this.isLoggable((System.Logger.Level)Objects.requireNonNull(level))) {
                this.log(level, (ResourceBundle)null, (String)msgSupplier.get(), (Object[])null);
            }
        }
        
        default public void log(System.Logger.Level level, Object obj) {
            Objects.requireNonNull(obj);
            if (this.isLoggable((System.Logger.Level)Objects.requireNonNull(level))) {
                this.log(level, (ResourceBundle)null, obj.toString(), (Object[])null);
            }
        }
        
        default public void log(System.Logger.Level level, String msg, Throwable thrown) {
            this.log(level, null, msg, thrown);
        }
        
        default public void log(System.Logger.Level level, Supplier<String> msgSupplier, Throwable thrown) {
            Objects.requireNonNull(msgSupplier);
            if (this.isLoggable((System.Logger.Level)Objects.requireNonNull(level))) {
                this.log(level, null, (String)msgSupplier.get(), thrown);
            }
        }
        
        default public void log(System.Logger.Level level, String format, Object... params) {
            this.log(level, null, format, params);
        }
        
        public abstract void log(System.Logger.Level p0, ResourceBundle p1, String p2, Throwable p3);
        
        public abstract void log(System.Logger.Level p0, ResourceBundle p1, String p2, Object... p3);
        
        public enum Level
        {
            ALL(Integer.MIN_VALUE), 
            TRACE(400), 
            DEBUG(500), 
            INFO(800), 
            WARNING(900), 
            ERROR(1000), 
            OFF(Integer.MAX_VALUE);
            
            private final int severity;
            
            private Level(int severity) {
                this.severity = severity;
            }
            
            public final String getName() {
                return this.name();
            }
            
            public final int getSeverity() {
                return this.severity;
            }
        }
    }
}
